@@ -28,6 +28,7 @@ import fnmatch
 import tempfile
 import shutil
 import functools
+import warnings
 
 # pattern -> (language, [keywords])
 XGETTEXT_CONFIG = {
@@ -46,6 +47,10 @@ class GettextError(Exception):
     pass
 
 
+class GettextWarning(Warning):
+    pass
+
+
 def _read_potfiles(src_root, potfiles):
     """Returns a list of paths for a POTFILES.in file"""
 
@@ -78,21 +83,26 @@ def get_pot_dependencies(po_dir):
     return _read_potfiles(src_root, potfiles_path)
 
 
-def _create_pot(potfiles_path, src_root, strict):
+def _get_pattern(path):
+    for pattern in XGETTEXT_CONFIG:
+        match_part = os.path.basename(path)
+        if match_part.endswith(".in"):
+            match_part = match_part.rsplit(".", 1)[0]
+        if fnmatch.fnmatch(match_part, pattern):
+            return pattern
+    return None
+
+
+def _create_pot(potfiles_path, src_root):
     potfiles = _read_potfiles(src_root, potfiles_path)
 
     groups = {}
     for path in potfiles:
-        for pattern in XGETTEXT_CONFIG:
-            match_part = os.path.basename(path)
-            if match_part.endswith(".in"):
-                match_part = match_part.rsplit(".", 1)[0]
-            if fnmatch.fnmatch(match_part, pattern):
-                groups.setdefault(pattern, []).append(path)
-                break
+        pattern = _get_pattern(path)
+        if pattern is not None:
+            groups.setdefault(pattern, []).append(path)
         else:
-            if strict:
-                raise ValueError("Unknown filetype: " + path)
+            raise ValueError("Unknown filetype: " + path)
 
     specs = []
     for pattern, paths in groups.items():
@@ -134,8 +144,8 @@ def _create_pot(potfiles_path, src_root, strict):
                 if p.returncode != 0:
                     raise GettextError(stderr
                                        or ("Got error: %d" % p.returncode))
-                if strict and stderr:
-                    raise GettextError(stderr)
+                if stderr:
+                    warnings.warn(stderr, GettextWarning)
             finally:
                 os.unlink(potfiles_in)
     except Exception:
@@ -146,15 +156,12 @@ def _create_pot(potfiles_path, src_root, strict):
 
 
 @contextlib.contextmanager
-def create_pot(po_dir, strict=False):
-    """Temporarily creates a .pot file in a temp directory.
-
-    If strict then error out on extraction warnings.
-    """
+def create_pot(po_dir):
+    """Temporarily creates a .pot file in a temp directory."""
 
     src_root = _src_root(po_dir)
     potfiles_path = os.path.join(po_dir, "POTFILES.in")
-    pot_path = _create_pot(potfiles_path, src_root, strict)
+    pot_path = _create_pot(potfiles_path, src_root)
     try:
         yield pot_path
     finally:
@@ -315,32 +322,35 @@ def get_missing(po_dir):
     src_root = _src_root(po_dir)
     potfiles_path = os.path.join(po_dir, "POTFILES.in")
     skip_path = os.path.join(po_dir, "POTFILES.skip")
-    potfiles = _read_potfiles(src_root, potfiles_path)
-    skipfiles = _read_potfiles(src_root, skip_path)
 
-    # generate a list of paths of files which are not marked translatable
+    # Generate a set of paths of files which are not marked translatable
     # and not skipped
-    potfiles = [os.path.relpath(p, src_root) for p in potfiles]
-    skipfiles = [os.path.relpath(p, src_root) for p in skipfiles]
-    not_translatable = []
+    pot_files = {os.path.relpath(p, src_root)
+                 for p in _read_potfiles(src_root, potfiles_path)}
+    skip_files = {os.path.relpath(p, src_root)
+                  for p in _read_potfiles(src_root, skip_path)}
+    not_translatable = set()
     for root, dirs, files in os.walk(src_root):
-        for dirname in list(dirs):
+        for dirname in dirs:
             dirpath = os.path.relpath(os.path.join(root, dirname), src_root)
-            if dirpath in skipfiles:
+            if dirpath in skip_files or dirpath.startswith("."):
                 dirs.remove(dirname)
 
         for name in files:
             path = os.path.relpath(os.path.join(root, name), src_root)
-            if path not in potfiles and path not in skipfiles:
-                not_translatable.append(path)
+            if path not in pot_files and path not in skip_files:
+                not_translatable.add(path)
 
-    # filter out any unknown filetypes
+    # Filter out any unknown filetypes
+    not_translatable = [p for p in not_translatable if _get_pattern(p) is not None]
+
+    # Filter out any files not containing translations
     fd, temp_path = tempfile.mkstemp("POTFILES.in")
     try:
         os.close(fd)
         _write_potfiles(src_root, temp_path, not_translatable)
 
-        pot_path = _create_pot(temp_path, src_root, strict=False)
+        pot_path = _create_pot(temp_path, src_root)
         try:
             infos = set()
             with open(pot_path, "r", encoding="utf-8") as h:
