@@ -1,5 +1,5 @@
 # Copyright 2005 Joe Wreschnig
-#           2017 Nick Boultbee
+#      2017-2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -10,7 +10,6 @@ import os
 import sys
 import threading
 import time
-from urllib.request import build_opener
 
 from gi.repository import Gtk, GLib, Pango, Gdk
 import feedparser
@@ -62,7 +61,7 @@ class Feed(list):
     def __fill_af(feed, af):
         try:
             af["title"] = feed.title or _("Unknown")
-        except:
+        except (TypeError, AttributeError):
             af["title"] = _("Unknown")
         try:
             af["date"] = "%04d-%02d-%02d" % feed.modified_parsed[:3]
@@ -78,7 +77,7 @@ class Feed(list):
                 ("license", "license")]:
             try:
                 value = getattr(feed, feedkey)
-            except:
+            except AttributeError:
                 pass
             else:
                 if value and value not in af.list(songkey):
@@ -136,9 +135,6 @@ class Feed(list):
 
     def parse(self):
         try:
-            if not self._check_feed():
-                return False
-
             doc = feedparser.parse(self.uri)
         except Exception as e:
             print_w("Couldn't parse feed: %s (%s)" % (self.uri, e))
@@ -158,7 +154,8 @@ class Feed(list):
         defaults = AudioFile({"feed": self.uri})
         try:
             self.__fill_af(doc.channel, defaults)
-        except:
+        except Exception as e:
+            print_w("Error creating feed data: %s (%s)" % (self.uri, e))
             return False
 
         entries = []
@@ -209,47 +206,16 @@ class Feed(list):
         self.__lastgot = time.time()
         return bool(uris)
 
-    def _check_feed(self):
-        """Validate stream a bit - failing fast where possible.
-
-           Constructs an equivalent(ish) HEAD request,
-           without re-writing feedparser completely.
-           (it never times out if reading from a stream - see #2257)"""
-        req = feedparser._build_urllib2_request(
-            self.uri, feedparser.USER_AGENT, None, None, None, None, {})
-        req.method = "HEAD"
-        opener = build_opener(feedparser._FeedURLHandler())
-        try:
-            result = opener.open(req)
-            ct_hdr = result.headers.get('Content-Type', "Unknown type")
-            content_type = ct_hdr.split(';')[0]
-            try:
-                status = result.status
-            except AttributeError:
-                print_w("Missing status code for feed %s" % self.uri)
-            else:
-                print_d("Pre-check: %s returned %s with content type '%s'" %
-                        (self.uri, status, content_type))
-                if content_type not in feedparser.ACCEPT_HEADER:
-                    print_w("Unusable content: %s. Perhaps %s is not a feed?" %
-                            (content_type, self.uri))
-                    return False
-                # No real need to check HTTP Status - errors are very unlikely
-                # to be a usable content type, and we should try to parse
-        finally:
-            opener.close()
-        return True
-
 
 class AddFeedDialog(GetStringDialog):
     def __init__(self, parent):
-        super(AddFeedDialog, self).__init__(
+        super().__init__(
             qltk.get_top_parent(parent), _("New Feed"),
             _("Enter the location of an audio feed:"),
             button_label=_("_Add"), button_icon=Icons.LIST_ADD)
 
     def run(self, text='', test=False):
-        uri = super(AddFeedDialog, self).run(text=text, test=test)
+        uri = super().run(text=text, test=test)
         if uri:
             if not isinstance(uri, str):
                 uri = uri.decode('utf-8')
@@ -386,7 +352,7 @@ class AudioFeeds(Browser):
         GLib.timeout_add(60 * 60 * 1000, klass.__do_check)
 
     def __init__(self, library):
-        super(AudioFeeds, self).__init__(spacing=6)
+        super().__init__(spacing=6)
         self.set_orientation(Gtk.Orientation.VERTICAL)
 
         self.__view = view = AllTreeView()
@@ -463,11 +429,7 @@ class AudioFeeds(Browser):
             self.__feeds.append(row=[feed])
             AudioFeeds.write()
         else:
-            ErrorMessage(
-                self, _("Unable to add feed"),
-                _("%s could not be added. The server may be down, "
-                  "or the location may not be an audio feed.") %
-                util.bold(util.escape(feed.uri))).run()
+            self.feed_error(feed).run()
 
     def __popup_menu(self, view):
         model, paths = view.get_selection().get_selected_rows()
@@ -522,11 +484,14 @@ class AudioFeeds(Browser):
                 self.__feeds.append(row=[feed])
                 AudioFeeds.write()
             else:
-                ErrorMessage(
-                    self, _("Unable to add feed"),
-                    _("%s could not be added. The server may be down, "
-                      "or the location may not be an audio feed.") %
-                    util.bold(util.escape(feed.uri))).run()
+                self.feed_error(feed).run()
+
+    def feed_error(self, feed: Feed) -> ErrorMessage:
+        return ErrorMessage(
+            self, _("Unable to add feed"),
+            _("%s could not be added. The server may be down, "
+              "or the location may not be an audio feed.") %
+            util.bold(util.escape(feed.uri)))
 
     def restore(self):
         try:
@@ -536,6 +501,7 @@ class AudioFeeds(Browser):
         else:
             self.__view.select_by_func(lambda r: r[0].name in names)
 
+
 browsers = []
 if not app.player or app.player.can_play_uri("http://"):
     browsers = [AudioFeeds]
