@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna,
-#           2011-2013,2016 Nick Boultbee
+#           2011-2020 Nick Boultbee
 #           2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,16 +22,17 @@ from quodlibet.qltk.browser import LibraryBrowser
 from quodlibet.qltk.properties import SongProperties
 from quodlibet.util.library import scan_library
 
-from quodlibet.order.repeat import RepeatListForever, RepeatSongForever, \
-        OneSong
+from quodlibet.order.repeat import RepeatListForever, RepeatSongForever, OneSong
 from quodlibet.order.reorder import OrderWeighted, OrderShuffle
 
+from quodlibet.config import RATINGS
+
 
 class CommandError(Exception):
     pass
 
 
-class CommandRegistry(object):
+class CommandRegistry:
     """Knows about all commands and handles parsing/executing them"""
 
     def __init__(self):
@@ -56,6 +56,7 @@ class CommandRegistry(object):
         def wrap(func):
             self._commands[name] = (func, args, optional)
             return func
+
         return wrap
 
     def handle_line(self, app, line):
@@ -83,6 +84,7 @@ class CommandRegistry(object):
             return self.run(app, command, *args)
         except CommandError as e:
             print_e(e)
+            util.print_exc()
         except:
             util.print_exc()
 
@@ -305,20 +307,30 @@ def _show_window(app):
     app.show()
 
 
-@registry.register("set-rating", args=1)
-def _set_rating(app, value):
+@registry.register("rating", args=1)
+def _rating(app, value):
     song = app.player.song
     if not song:
         return
 
-    value = arg2text(value)
-
-    try:
-        song["~#rating"] = max(0.0, min(1.0, float(value)))
-    except (ValueError, TypeError):
-        pass
+    if value[0] in ('+', '-'):
+        if len(value) > 1:
+            try:
+                change = float(value[1:])
+            except ValueError:
+                return
+        else:
+            change = (1 / RATINGS.number)
+        if value[0] == '-':
+            change = -change
+        rating = song["~#rating"] + change
     else:
-        app.library.changed([song])
+        try:
+            rating = float(value)
+        except (ValueError, TypeError):
+            return
+    song["~#rating"] = max(0.0, min(1.0, rating))
+    app.library.changed([song])
 
 
 @registry.register("dump-browsers")
@@ -543,7 +555,8 @@ def _print_playing(app, fstring=None):
     if song is None:
         song = AudioFile({"~filename": fsnative(u"/")})
         song.sanitize()
-
+    else:
+        song = app.player.with_elapsed_info(song)
     return text2fsn(Pattern(fstring).format(song) + u"\n")
 
 
