@@ -15,6 +15,7 @@
 import os
 import threading
 import time
+from typing import Dict, List
 from hashlib import md5
 from urllib.parse import urlencode
 
@@ -79,7 +80,7 @@ def config_get_artist_pattern():
     return plugin_config.get('artistpat') or DEFAULT_ARTISTPAT
 
 
-class QLSubmitQueue(object):
+class QLSubmitQueue:
     """Manages the submit queue for scrobbles. Works independently of the
     QLScrobbler plugin being enabled; other plugins may use submit() to queue
     songs for scrobbling.
@@ -92,7 +93,7 @@ class QLSubmitQueue(object):
 
     # These objects are shared across instances, to allow other plugins to
     # queue scrobbles in future versions of QL
-    queue = []
+    queue: List[Dict[str, str]] = []
     changed_event = threading.Event()
 
     def set_nowplaying(self, song):
