@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
+#           2018 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 import shutil
@@ -11,7 +12,7 @@ import shutil
 from gi.repository import Gtk
 
 import quodlibet
-from quodlibet import _
+from quodlibet import _, print_w
 from quodlibet import app
 from quodlibet import config
 from quodlibet.plugins.events import EventPlugin
@@ -19,8 +20,8 @@ from quodlibet.qltk import Icons
 
 
 def get_path():
-    out = os.path.join(quodlibet.get_user_dir(), "current.cover")
-    return config.get("plugins", __name__, out)
+    default = os.path.join(quodlibet.get_user_dir(), "current.cover")
+    return config.get("plugins", __name__, default=default)
 
 
 def set_path(value):
@@ -34,19 +35,19 @@ class PictureSaver(EventPlugin):
     PLUGIN_ICON = Icons.DOCUMENT_SAVE
 
     def plugin_on_song_started(self, song):
-        outfile = get_path()
-        if song is None:
+        def delete(outfile):
             try:
                 os.unlink(outfile)
-            except EnvironmentError:
-                pass
+            except EnvironmentError as e:
+                print_w("Couldn't delete '%s' (%s)" % (outfile, e))
+
+        outfile = get_path()
+        if song is None:
+            delete(outfile)
         else:
             cover = app.cover_manager.get_cover(song)
             if cover is None:
-                try:
-                    os.unlink(outfile)
-                except EnvironmentError:
-                    pass
+                delete(outfile)
             else:
                 with open(outfile, "wb") as f:
                     f.write(cover.read())
