@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from contextlib import contextmanager
 
@@ -22,7 +22,7 @@ from quodlibet.util import print_w
 from quodlibet.qltk.ccb import ConfigCheckButton
 
 
-class Status(object):
+class Status:
     QUEUED = 0
     ANALYZING = 1
     LOOKUP = 2
@@ -46,7 +46,7 @@ class Status(object):
             return _("Unknown")
 
 
-class SearchEntry(object):
+class SearchEntry:
 
     def __init__(self, song):
         self.song = song
@@ -108,7 +108,7 @@ class SearchEntry(object):
 class ResultView(AllTreeView):
 
     def __init__(self):
-        super(ResultView, self).__init__()
+        super().__init__()
 
         self._release_ids = {}
 
@@ -176,7 +176,7 @@ class ResultView(AllTreeView):
             render.set_property('ellipsize', Pango.EllipsizeMode.END)
             column = Gtk.TreeViewColumn(util.tag(tag), render)
 
-            def cell_data(column, cell, model, iter_, data, tag=tag):
+            def cell_data2(column, cell, model, iter_, data, tag=tag):
                 entry = model.get_value(iter_)
                 release = entry.release
                 if not release:
@@ -186,7 +186,7 @@ class ResultView(AllTreeView):
                     value = ", ".join(value.split("\n"))
                     cell.set_property("text", value)
 
-            column.set_cell_data_func(render, cell_data)
+            column.set_cell_data_func(render, cell_data2)
             self.append_column(column)
             if tag == "tracknumber":
                 self._track_column = column
@@ -241,7 +241,7 @@ def score_release(release):
 class SearchWindow(Window):
 
     def __init__(self, songs, title=None):
-        super(SearchWindow, self).__init__(
+        super().__init__(
             default_width=800, default_height=400, border_width=12,
             title=title)
 
