@@ -12,6 +12,7 @@ import time
 from datetime import date
 from threading import Thread
 from urllib.parse import urlencode
+import json
 
 from gi.repository import Gtk, GLib
 
@@ -23,10 +24,6 @@ from quodlibet.qltk import Icons
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.urllib import urlopen
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
 
 API_KEY = "f536cdadb4c2aec75ae15e2b719cb3a1"
 
@@ -60,7 +57,7 @@ def config_get(key, default=None):
     return config.get('plugins', 'lastfmsync_%s' % key, default)
 
 
-class LastFMSyncCache(object):
+class LastFMSyncCache:
     """Stores the Last.fm charts for a particular user."""
 
     registered = 0
@@ -213,7 +210,7 @@ class LastFMSyncCache(object):
 
 class LastFMSyncWindow(qltk.Dialog):
     def __init__(self, parent):
-        super(LastFMSyncWindow, self).__init__(
+        super().__init__(
                 _("Last.fm Sync"), parent)
         self.add_button(_("_Cancel"), Gtk.ResponseType.REJECT)
         self.add_icon_button(_("_Save"), Icons.DOCUMENT_SAVE,
