@@ -1,34 +1,37 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
 #           2012-2017 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 # Much of this code is highly optimized, because many of the functions
 # are called in tight loops. Don't change things just to make them
 # more readable, unless they're also faster.
 
 import os
+import re
 import shutil
 import time
+from typing import List
+from collections import OrderedDict
+from itertools import zip_longest
 
 from senf import fsn2uri, fsnative, fsn2text, devnull, bytes2fsn, path2fsn
 
-from quodlibet import _
+from quodlibet import _, print_d
 from quodlibet import util
 from quodlibet import config
-from quodlibet.util.path import mkdir, mtime, expanduser, \
-    normalize_path, escape_filename, ismount
+from quodlibet.util.path import mkdir, mtime, expanduser, normalize_path, \
+                                ismount, get_home_dir, RootPathFile
 from quodlibet.util.string import encode, decode, isascii
+from quodlibet.util.environment import is_windows
 
 from quodlibet.util import iso639
 from quodlibet.util import human_sort_key as human, capitalize
 
 from quodlibet.util.tags import TAG_ROLES, TAG_TO_SORT
-from quodlibet.compat import iteritems, string_types, text_type, \
-    number_types, listitems, izip_longest, integer_types, PY3, listfilter
 
 from ._image import ImageContainer
 from ._misc import AudioFileError, translate_errors
@@ -59,7 +62,7 @@ NUMERIC_ZERO_DEFAULT.update(SIZE_TAGS)
 FILESYSTEM_TAGS = {"~filename", "~basename", "~dirname", "~mountpoint"}
 """Values are bytes in Linux instead of unicode"""
 
-SORT_TO_TAG = dict([(v, k) for (k, v) in iteritems(TAG_TO_SORT)])
+SORT_TO_TAG = dict((v, k) for (k, v) in TAG_TO_SORT.items())
 """Reverse map, so sort tags can fall back to the normal ones"""
 
 PEOPLE_SORT = [TAG_TO_SORT.get(k, k) for k in PEOPLE]
@@ -82,8 +85,8 @@ def decode_value(tag, value):
         if isinstance(value, float):
             return u"%.2f" % value
         else:
-            return text_type(value)
-    return text_type(value)
+            return str(value)
+    return str(value)
 
 
 class AudioFile(dict, ImageContainer):
@@ -124,7 +127,10 @@ class AudioFile(dict, ImageContainer):
     format = "Unknown Audio File"
     """The underlying file format"""
 
-    mimes = []
+    supports_rating_and_play_count_in_file = False
+    """Does this format support storing ratings and play counts in the file"""
+
+    mimes: List[str] = []
     """MIME types this class can represent"""
 
     def __init__(self, default=tuple(), **kwargs):
@@ -134,7 +140,7 @@ class AudioFile(dict, ImageContainer):
             self[key] = value
 
     def __song_key(self):
-        return (self("~#disc", 1), self("~#track", 1),
+        return (self("~#disc", 0), self("~#track", 0),
             human(self("artistsort")),
             self.get("musicbrainz_artistid", ""),
             human(self.get("title", "")),
@@ -165,7 +171,7 @@ class AudioFile(dict, ImageContainer):
         elif tag in FILESYSTEM_TAGS:
             return lambda song: fsn2text(song(tag))
         elif tag.startswith("~#") and "~" not in tag[2:]:
-            return lambda song: song(tag)
+            return lambda song: song(tag, 0)
         return lambda song: human(song(tag))
 
     def __getstate__(self):
@@ -178,34 +184,18 @@ class AudioFile(dict, ImageContainer):
 
     def __setitem__(self, key, value):
         # validate key
-        if PY3:
-            if not isinstance(key, text_type):
-                raise TypeError("key has to be str")
-        else:
-            if isinstance(key, text_type):
-                # we try to save keys as encoded ASCII to save memory
-                # under PY2. Everything else besides ASCII combined with
-                # unicode breaks hashing even if the default encoding
-                # it utf-8.
-                try:
-                    key = key.encode("ascii")
-                except UnicodeEncodeError:
-                    pass
-            elif isinstance(key, bytes):
-                # make sure we set ascii keys only
-                key.decode("ascii")
-            else:
-                raise TypeError("key needs to be unicode or ASCII str")
+        if not isinstance(key, str):
+            raise TypeError("key has to be str")
 
         # validate value
         if key.startswith("~#"):
-            if not isinstance(value, number_types):
+            if not isinstance(value, (int, float)):
                 raise TypeError
         elif key in FILESYSTEM_TAGS:
             if not isinstance(value, fsnative):
                 value = path2fsn(value)
         else:
-            value = text_type(value)
+            value = str(value)
 
         dict.__setitem__(self, key, value)
 
@@ -245,6 +235,13 @@ class AudioFile(dict, ImageContainer):
     def __ne__(self, other):
         return self is not other
 
+    def has_rating_and_playcount_in_file(self, email):
+        """Returns True if the audio file has rating and play counts for the
+        provided email. Returns False otherwise, or if storing ratings and play
+        counts in the file is not supported for this audio format.
+        """
+        return False
+
     def reload(self):
         """Reload an audio file from disk. If reloading fails nothing will
         change.
@@ -273,7 +270,7 @@ class AudioFile(dict, ImageContainer):
         """Returns a list of keys that are not internal, i.e. they don't
         have '~' in them."""
 
-        return listfilter(lambda s: s[:1] != "~", self.keys())
+        return list(filter(lambda s: s[:1] != "~", self.keys()))
 
     def prefixkeys(self, prefix):
         """Returns a list of dict keys that either match prefix or start
@@ -291,7 +288,7 @@ class AudioFile(dict, ImageContainer):
         return "\n".join(self.list_unique(sorted(self.prefixkeys(tag))))
 
     def iterrealitems(self):
-        return ((k, v) for (k, v) in iteritems(self) if k[:1] != "~")
+        return ((k, v) for (k, v) in self.items() if k[:1] != "~")
 
     def __call__(self, key, default=u"", connector=" - ", joiner=', '):
         """Return the value(s) for a key, synthesizing if necessary.
@@ -390,7 +387,7 @@ class AudioFile(dict, ImageContainer):
                 except KeyError:
                     return fsn2uri(self["~filename"])
             elif key == "format":
-                return self.get("~format", text_type(self.format))
+                return self.get("~format", str(self.format))
             elif key == "codec":
                 codec = self.get("~codec")
                 if codec is None:
@@ -440,18 +437,27 @@ class AudioFile(dict, ImageContainer):
             elif key == "lyrics":
                 # First, try the embedded lyrics.
                 try:
-                    return self[key]
+                    return self["lyrics"]
+                except KeyError:
+                    pass
+
+                try:
+                    return self["unsyncedlyrics"]
                 except KeyError:
                     pass
 
                 # If there are no embedded lyrics, try to read them from
                 # the external file.
                 try:
-                    fileobj = open(self.lyric_filename, "rU")
+                    with open(self.lyric_filename, "rb") as fileobj:
+                        print_d("Reading lyrics from %s" % self.lyric_filename)
+                        text = fileobj.read().decode("utf-8", "replace")
+                        # try to skip binary files
+                        if "\0" in text:
+                            return default
+                        return text
                 except EnvironmentError:
                     return default
-                else:
-                    return fileobj.read().decode("utf-8", "replace")
             elif key == "filesize":
                 return util.format_size(self("~#filesize", 0))
             elif key == "playlists":
@@ -459,7 +465,7 @@ class AudioFile(dict, ImageContainer):
                 # Avoid circular references from formats/__init__.py
                 from quodlibet.util.collection import Playlist
                 playlists = Playlist.playlists_featuring(self)
-                return "\n".join([s.name for s in playlists]) or default
+                return "\n".join(s.name for s in playlists) or default
             elif key.startswith("#replaygain_"):
                 try:
                     val = self.get(key[1:], default)
@@ -507,10 +513,18 @@ class AudioFile(dict, ImageContainer):
 
         role_map = {}
         for key in role_tag_keys:
-            if key != role_tag:
-                role = key.rsplit(":", 1)[-1]
-                for name in self.list(key):
-                    role_map.setdefault(name, []).append(role)
+            if key == role_tag:
+                # #2986: don't add a role description for the bare tag, unless
+                # this is a composite tag (e.g. only show "(Performance)" for
+                # ~people:roles and not ~performer:roles).
+                if sub_keys is None:
+                    continue
+                else:
+                    role = TAG_ROLES.get(role_tag, role_tag)
+            else:
+                role = key.split(":", 1)[-1]
+            for name in self.list(key):
+                role_map.setdefault(name, []).append(role)
 
         if sub_keys is None:
             names = self.list_unique(role_tag_keys)
@@ -534,24 +548,127 @@ class AudioFile(dict, ImageContainer):
 
     @property
     def lyric_filename(self):
-        """Returns the (potential) lyrics filename for this file"""
+        """Returns the validated, or default, lyrics filename for this
+        file. User defined '[memory] lyric_rootpaths' and
+        '[memory] lyric_filenames' matches take precedence"""
 
-        filename = self.comma("title").replace(u'/', u'')[:128] + u'.lyric'
-        sub_dir = ((self.comma("lyricist") or self.comma("artist"))
-                  .replace(u'/', u'')[:128])
+        from quodlibet.pattern \
+            import ArbitraryExtensionFileFromPattern as expand_patterns
 
-        if os.name == "nt":
-            # this was added at a later point. only use escape_filename here
-            # to keep the linux case the same as before
-            filename = escape_filename(filename)
-            sub_dir = escape_filename(sub_dir)
-        else:
-            filename = fsnative(filename)
-            sub_dir = fsnative(sub_dir)
+        rx_params = re.compile(r'[^\\]<[^' + re.escape(os.sep) + r']*[^\\]>')
 
-        path = os.path.join(
-            expanduser(fsnative(u"~/.lyrics")), sub_dir, filename)
-        return path
+        def expand_pathfile(rpf):
+            """Return the expanded RootPathFile"""
+            expanded = []
+            root = expanduser(rpf.root)
+            pathfile = expanduser(rpf.pathfile)
+            if rx_params.search(pathfile):
+                root = expand_patterns(root).format(self)
+                pathfile = expand_patterns(pathfile).format(self)
+            rpf = RootPathFile(root, pathfile)
+            expanded.append(rpf)
+            if not os.path.exists(pathfile) and is_windows():
+                # prioritise a special character encoded version
+                #
+                # most 'alien' chars are supported for 'nix fs paths, and we
+                # only pass the proposed path through 'escape_filename' (which
+                # apparently doesn't respect case) if we don't care about case!
+                #
+                # FIX: assumes 'nix build used on a case-sensitive fs, nt case
+                # insensitive. clearly this is not biting anyone though (yet!)
+                pathfile = os.path.sep.join([rpf.root, rpf.end_escaped])
+                rpf = RootPathFile(rpf.root, pathfile)
+                expanded.insert(len(expanded) - 1, rpf)
+            return expanded
+
+        def sanitise(sep, parts):
+            """Return a santisied version of a path's parts"""
+            return sep.join(part.replace(os.path.sep, u'')[:128]
+                                for part in parts)
+
+        # setup defaults (user-defined take precedence)
+        # root search paths
+        lyric_paths = \
+            config.getstringlist("memory", "lyric_rootpaths", [])
+        # ensure default paths
+        lyric_paths.append(os.path.join(get_home_dir(), ".lyrics"))
+        lyric_paths.append(
+            os.path.join(os.path.dirname(self.comma('~filename'))))
+        # search pathfile names
+        lyric_filenames = \
+            config.getstringlist("memory", "lyric_filenames", [])
+        # ensure some default pathfile names
+        lyric_filenames.append(
+            sanitise(os.sep, [(self.comma("lyricist") or
+                              self.comma("artist")),
+                              self.comma("title")]) + u'.lyric')
+        lyric_filenames.append(
+            sanitise(' - ', [(self.comma("lyricist") or
+                             self.comma("artist")),
+                             self.comma("title")]) + u'.lyric')
+
+        # generate all potential paths (unresolved/unexpanded)
+        pathfiles = OrderedDict()
+        for r in lyric_paths:
+            for f in lyric_filenames:
+                pathfile = os.path.join(r, os.path.dirname(f),
+                                        fsnative(os.path.basename(f)))
+                rpf = RootPathFile(r, pathfile)
+                if not pathfile in pathfiles:
+                    pathfiles[pathfile] = rpf
+
+        #print_d("searching for lyrics in:\n%s" % '\n'.join(pathfiles.keys()))
+
+        # expand each raw pathfile in turn and test for existence
+        match_ = ""
+        pathfiles_expanded = OrderedDict()
+        for pf, rpf in pathfiles.items():
+            for rpf in expand_pathfile(rpf):  # resolved as late as possible
+                pathfile = rpf.pathfile
+                pathfiles_expanded[pathfile] = rpf
+                if os.path.exists(pathfile):
+                    match_ = pathfile
+                    break
+            if match_ != "":
+                break
+
+        if not match_:
+            # search even harder!
+            lyric_extensions = ['lyric', 'lyrics', '', 'txt']
+            #print_d("extending search to extensions: %s" % lyric_extensions)
+
+            def generate_mod_ext_paths(pathfile):
+                # separate pathfile's extension (if any)
+                ext = os.path.splitext(pathfile)[1][1:]
+                path = pathfile[:-1 * len(ext)].strip('.') if ext else pathfile
+                # skip the proposed lyric extension if it is the same as
+                # the original for a given search pathfile stub - it has
+                # already been tested without success!
+                extra_extensions = [x for x in lyric_extensions if x != ext]
+
+                # join valid new extensions to pathfile stub and return
+                return ['.'.join([path, ext]) if ext else path
+                           for ext in extra_extensions]
+
+            # look for a match by modifying the extension for each of the
+            # (now fully resolved) 'pathfiles_expanded' search items
+            for pathfile in pathfiles_expanded.keys():
+                # get alternatives for existence testing
+                paths_mod_ext = generate_mod_ext_paths(pathfile)
+                for path_ext in paths_mod_ext:
+                    if os.path.exists(path_ext):
+                        # persistence has paid off!
+                        #print_d("extended search match!")
+                        match_ = path_ext
+                        break
+                if match_:
+                    break
+
+        if not match_:
+            # default
+            match_ = list(pathfiles_expanded.keys())[0]
+
+        return match_
 
     @property
     def has_rating(self):
@@ -583,7 +700,7 @@ class AudioFile(dict, ImageContainer):
         else:
             v = self.get(key, u"")
 
-        if isinstance(v, number_types):
+        if isinstance(v, (int, float)):
             return v
         else:
             return v.replace("\n", ", ")
@@ -604,7 +721,7 @@ class AudioFile(dict, ImageContainer):
             if v == "":
                 return []
             else:
-                return v.split("\n") if isinstance(v, text_type) else [v]
+                return v.split("\n") if isinstance(v, str) else [v]
         else:
             v = self.get(key)
             return [] if v is None else v.split("\n")
@@ -628,7 +745,7 @@ class AudioFile(dict, ImageContainer):
             sort = sort.split("\n") if sort else []
 
         result = []
-        for d, s in izip_longest(display, sort):
+        for d, s in zip_longest(display, sort):
             if d is not None:
                 result.append((d, (s if s is not None and s != "" else d)))
         return result
@@ -669,7 +786,7 @@ class AudioFile(dict, ImageContainer):
 
         merged = AudioFile()
         text = {}
-        for key, value in iteritems(self):
+        for key, value in self.items():
             lower = key.lower()
             if key.startswith("~#"):
                 merged[lower] = value
@@ -751,9 +868,9 @@ class AudioFile(dict, ImageContainer):
         """
 
         # Replace nulls with newlines, trimming zero-length segments
-        for key, val in listitems(self):
+        for key, val in list(self.items()):
             self[key] = val
-            if isinstance(val, string_types) and '\0' in val:
+            if isinstance(val, str) and '\0' in val:
                 self[key] = '\n'.join(filter(lambda s: s, val.split('\0')))
             # Remove unnecessary defaults
             if key in NUMERIC_ZERO_DEFAULT and val == 0:
@@ -810,14 +927,14 @@ class AudioFile(dict, ImageContainer):
         """
 
         def encode_key(k):
-            return encode(k) if isinstance(k, text_type) else k
+            return encode(k) if isinstance(k, str) else k
 
         s = []
         for k in self.keys():
             enc_key = encode_key(k)
             assert isinstance(enc_key, bytes)
 
-            if isinstance(self[k], integer_types):
+            if isinstance(self[k], int):
                 l = enc_key + encode("=%d" % self[k])
                 s.append(l)
             elif isinstance(self[k], float):
@@ -845,23 +962,11 @@ class AudioFile(dict, ImageContainer):
             text (bytes)
         """
 
-        def decode_key(key):
-            """str if ascii, otherwise decode using utf-8"""
-
-            if PY3:
-                return decode(key)
-
-            try:
-                key.decode("ascii")
-            except ValueError:
-                return decode(key)
-            return key
-
         for line in text.split(b"\n"):
             if not line:
                 continue
             parts = line.split(b"=")
-            key = decode_key(parts[0])
+            key = decode(parts[0])
             val = b"=".join(parts[1:])
             if key == "~format":
                 pass
@@ -928,7 +1033,7 @@ class AudioFile(dict, ImageContainer):
         used when the song does not have replaygain information.
         """
         for profile in profiles:
-            if profile is "none":
+            if profile == "none":
                 return 1.0
             try:
                 db = float(self["replaygain_%s_gain" % profile].split()[0])
@@ -937,14 +1042,22 @@ class AudioFile(dict, ImageContainer):
                 continue
             else:
                 db += pre_amp_gain
-                scale = 10. ** (db / 20)
-                if scale * peak > 1:
-                    scale = 1.0 / peak  # don't clip
+                try:
+                    scale = 10. ** (db / 20)
+                except OverflowError:
+                    scale = 1.0 / peak
+                else:
+                    if scale * peak > 1:
+                        scale = 1.0 / peak  # don't clip
                 return min(15, scale)
         else:
-            scale = 10. ** ((fallback_gain + pre_amp_gain) / 20)
-            if scale > 1:
-                scale = 1.0  # don't clip
+            try:
+                scale = 10. ** ((fallback_gain + pre_amp_gain) / 20)
+            except OverflowError:
+                scale = 1.0
+            else:
+                if scale > 1:
+                    scale = 1.0  # don't clip
             return min(15, scale)
 
     def write(self):
