@@ -1,11 +1,11 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2010 Joe Wreschnig, Michael Urman
 # Copyright 2010,2013 Christoph Reiter
 # Copyright 2013-2015 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 # Pattern := (<String> | <Tags>)*
 # String := ([^<>|\\]|\\.)+, a string
@@ -13,15 +13,15 @@
 
 import os
 import re
-from re import Scanner
+from re import Scanner  # type: ignore
+from urllib.parse import quote_plus
+
+from senf import sep, fsnative, expanduser
 
 from quodlibet import util
 from quodlibet.query import Query
-from quodlibet.compat import exec_, itervalues
-from quodlibet.util.path import expanduser, fsnative, sep
 from quodlibet.util.path import strip_win32_incompat_from_path, limit_path
-from quodlibet.formats._audio import decode_value
-from quodlibet.compat import quote_plus
+from quodlibet.formats._audio import decode_value, FILESYSTEM_TAGS
 
 # Token types.
 (OPEN, CLOSE, TEXT, COND, EOF) = range(5)
@@ -39,7 +39,7 @@ class LexerError(error):
     pass
 
 
-class PatternLexeme(object):
+class PatternLexeme:
     _reverse = {OPEN: "OPEN", CLOSE: "CLOSE", TEXT: "TEXT", COND: "COND",
                 EOF: "EOF"}
 
@@ -48,7 +48,7 @@ class PatternLexeme(object):
         self.lexeme = lexeme
 
     def __repr__(self):
-        return (super(PatternLexeme, self).__repr__().split()[0] +
+        return (super().__repr__().split()[0] +
                 " type=" + repr(self.type) + " (" +
                 str(self._reverse[self.type]) +
                 "), lexeme=" + repr(self.lexeme) + ">")
@@ -77,7 +77,7 @@ class PatternLexer(Scanner):
             return iter(s[0] + [PatternLexeme(EOF, "")])
 
 
-class PatternNode(object):
+class PatternNode:
     def __init__(self):
         self.children = []
 
@@ -85,7 +85,7 @@ class PatternNode(object):
         return "Pattern(%s)" % (", ".join(map(repr, self.children)))
 
 
-class TextNode(object):
+class TextNode:
     def __init__(self, text):
         self.text = text
 
@@ -93,7 +93,7 @@ class TextNode(object):
         return "Text(\"%s\")" % self.text
 
 
-class ConditionNode(object):
+class ConditionNode:
     def __init__(self, expr, ifcase, elsecase):
         self.expr = expr
         self.ifcase = ifcase
@@ -104,7 +104,7 @@ class ConditionNode(object):
         return "Condition(expression: \"%s\", if: %s, else: %s)" % (t, i, e)
 
 
-class TagNode(object):
+class TagNode:
     def __init__(self, tag):
         self.tag = tag
 
@@ -112,7 +112,7 @@ class TagNode(object):
         return "Tag(\"%s\")" % self.tag
 
 
-class PatternParser(object):
+class PatternParser:
     def __init__(self, tokens):
         self.tokens = iter(tokens)
         self.lookahead = next(self.tokens)
@@ -134,7 +134,7 @@ class PatternParser(object):
         tag = self.lookahead.lexeme
         # fix bad tied tags
         if tag[:1] != "~" and "~" in tag:
-                tag = "~" + tag
+            tag = "~" + tag
         try:
             self.match(TEXT)
         except ParseError:
@@ -175,13 +175,13 @@ class PatternParser(object):
             if self.lookahead.type in tokens:
                 self.lookahead = next(self.tokens)
             else:
-                raise ParseError("The token '%s' is not the type exected." % (
+                raise ParseError("The token '%s' is not the type expected." % (
                     self.lookahead.lexeme))
         except StopIteration:
             self.lookahead = PatternLexeme(EOF, "")
 
 
-class PatternFormatter(object):
+class PatternFormatter:
     _format = None
     _post = None
     _text = None
@@ -195,17 +195,19 @@ class PatternFormatter(object):
     class Dummy(dict):
 
         def __call__(self, key, *args):
+            if key in FILESYSTEM_TAGS:
+                return fsnative(u"_")
             if key[:2] == "~#" and "~" not in key[2:]:
                 return 0
             return u"_"
 
-        def comma(self, *args):
+        def comma(self, key, *args):
             return u"_"
 
-        def list_separate(self, *args):
+        def list_separate(self, key):
             return [u""]
 
-    class SongProxy(object):
+    class SongProxy:
         def __init__(self, realsong, formatter):
             self.__song = realsong
             self.__formatter = formatter
@@ -218,7 +220,8 @@ class PatternFormatter(object):
 
         def comma(self, key):
             value = self.__song.comma(key)
-            value = decode_value(key, value)
+            if isinstance(value, (int, float)):
+                value = decode_value(key, value)
             if self.__formatter:
                 return self.__formatter(key, value)
             return value
@@ -267,7 +270,7 @@ class PatternFormatter(object):
     __mod__ = format
 
 
-class PatternCompiler(object):
+class PatternCompiler:
     def __init__(self, root):
         self.__root = root.node
 
@@ -284,10 +287,10 @@ class PatternCompiler(object):
         content.append("  return r")
         code = "\n".join(content)
 
-        scope = dict(itervalues(queries))
+        scope = dict(queries.values())
         if text_formatter:
             scope["_format"] = text_formatter
-        exec_(compile(code, "<string>", "exec"), scope)
+        exec(compile(code, "<string>", "exec"), scope)
         return scope["f"], tags
 
     def __get_value(self, text, scope, tag):
@@ -401,7 +404,7 @@ class _FileFromPattern(PatternFormatter):
 
     def _post(self, value, song, keep_extension=True):
         if value:
-            assert isinstance(value, unicode)
+            assert isinstance(value, str)
             value = fsnative(value)
 
             if keep_extension:
@@ -412,7 +415,7 @@ class _FileFromPattern(PatternFormatter):
                     value += ext.lower()
 
             if os.name == "nt":
-                assert isinstance(value, unicode)
+                assert isinstance(value, str)
                 value = strip_win32_incompat_from_path(value)
 
             value = expanduser(value)
@@ -420,7 +423,9 @@ class _FileFromPattern(PatternFormatter):
 
             if sep in value and not os.path.isabs(value):
                 raise ValueError("Pattern is not rooted")
-        return value
+            return value
+        else:
+            return fsnative(value)
 
 
 class _ArbitraryExtensionFileFromPattern(_FileFromPattern):
@@ -444,6 +449,8 @@ def replace_nt_seps(string):
 
 
 def FileFromPattern(string):
+    """Gives fsnative, not unicode"""
+
     return Pattern(replace_nt_seps(string), _FileFromPattern)
 
 
@@ -475,10 +482,10 @@ class _XMLFromMarkupPattern(_XMLFromPattern):
 
 
 def XMLFromMarkupPattern(string):
-    """Like XMLFromPattern but allows using [] instead of \<\> for
+    """Like XMLFromPattern but allows using [] instead of \\<\\> for
     pango markup to get rid of all the escaping in the common case.
 
-    To get text like "[b]" escape the first '[' like "\[b]"
+    To get text like "[b]" escape the first '[' like "\\[b]"
     """
 
     return Pattern(string, _XMLFromMarkupPattern)
