@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2011 Joe Wreschnig, Michael Urman, Steven Robertson,
 #           2011-2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import gi
 try:
@@ -24,13 +24,14 @@ from quodlibet import _
 from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
     MainRunnerAbortedError, MainRunnerTimeoutError, print_w, print_d, \
     print_e, print_
+from quodlibet.util.path import uri2gsturi
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.notif import Task
-from quodlibet.compat import iteritems
+from quodlibet.formats.mod import ModFile
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
-    GStreamerSink, link_many, bin_debug)
+                   GStreamerSink, link_many, bin_debug, AudioSinks)
 from .plugins import GStreamerPluginHandler
 from .prefs import GstPlayerPreferences
 
@@ -40,7 +41,7 @@ STATE_CHANGE_TIMEOUT = Gst.SECOND * 4
 const.MinVersions.GSTREAMER.check(Gst.version())
 
 
-class BufferingWrapper(object):
+class BufferingWrapper:
     """A wrapper for a Gst.Element (usually GstPlayBin) which pauses the
     elmement in case buffering is needed, but hides the fact that it does.
 
@@ -151,8 +152,15 @@ class BufferingWrapper(object):
         self.bin = None
 
 
-def sink_has_external_state(sink):
-    return sink.get_factory().get_name() == "pulsesink"
+def sink_has_external_state(sink: Gst.Element) -> bool:
+    try:
+        sink_type = AudioSinks(sink.get_factory().get_name())
+    except TypeError:
+        return False
+    if sink_type == AudioSinks.WASAPI:
+        # https://bugzilla.gnome.org/show_bug.cgi?id=796386
+        return hasattr(sink.props, "volume")
+    return sink_type == AudioSinks.PULSE
 
 
 def sink_state_is_valid(sink):
@@ -165,6 +173,144 @@ def sink_state_is_valid(sink):
     return current_state >= Gst.State.PAUSED
 
 
+class Seeker:
+    """Manages async seeking and position reporting for a pipeline.
+
+    You have to call destroy() before it gets gc'd
+    """
+
+    def __init__(self, playbin, player):
+        self._player = player
+        self._playbin = playbin
+
+        self._last_position = 0
+        self._seek_requests = []
+        self._active_seeks = []
+        self._seekable = False
+
+        bus = playbin.get_bus()
+        bus.add_signal_watch()
+        self._bus_id = bus.connect('message', self._on_message)
+        player.notify("seekable")
+
+    @property
+    def seekable(self):
+        """If the current stream is seekable"""
+
+        return self._seekable
+
+    def destroy(self):
+        """This needs to be called before it gets GC'ed"""
+
+        del self._seek_requests[:]
+        del self._active_seeks[:]
+
+        if self._bus_id:
+            bus = self._playbin.get_bus()
+            bus.disconnect(self._bus_id)
+            bus.remove_signal_watch()
+            self._bus_id = None
+
+        self._player = None
+        self._playbin = None
+
+    def set_position(self, pos):
+        """Set the position. Async and might not succeed.
+
+        Args:
+            pos (int): position in milliseconds
+        """
+
+        pos = max(0, int(pos))
+
+        # We need at least a paused state to seek, if there is non active
+        # or pending, request one async.
+        res, next_state, pending = self._playbin.get_state(timeout=0)
+        if pending != Gst.State.VOID_PENDING:
+            next_state = pending
+        if next_state < Gst.State.PAUSED:
+            self._playbin.set_state(Gst.State.PAUSED)
+
+        self._set_position(self._player.song, pos)
+
+    def get_position(self):
+        """Get the position
+
+        Returns:
+            int: the position in milliseconds
+        """
+
+        if self._seek_requests:
+            self._last_position = self._seek_requests[-1][1]
+        elif self._active_seeks:
+            self._last_position = self._active_seeks[-1][1]
+        else:
+            # While we are actively seeking return the last wanted position.
+            # query_position() returns 0 while in this state
+            ok, p = self._playbin.query_position(Gst.Format.TIME)
+            if ok:
+                p //= Gst.MSECOND
+                # During stream seeking querying the position fails.
+                # Better return the last valid one instead of 0.
+                self._last_position = p
+
+        return self._last_position
+
+    def reset(self):
+        """In case the underlying stream has changed, call this to
+        abort any pending seeking actions and update the seekable state
+        """
+
+        self._last_position = 0
+        del self._seek_requests[:]
+        del self._active_seeks[:]
+        self._refresh_seekable()
+
+    def _refresh_seekable(self):
+        query = Gst.Query.new_seeking(Gst.Format.TIME)
+        if self._playbin.query(query):
+            seekable = query.parse_seeking()[1]
+        elif self._player.song is None:
+            seekable = False
+        else:
+            seekable = True
+
+        if seekable != self._seekable:
+            self._seekable = seekable
+            self._player.notify("seekable")
+
+    def _on_message(self, bus, message):
+        if message.type == Gst.MessageType.ASYNC_DONE:
+            # we only get one ASYNC_DONE for multiple seeks, so flush all
+
+            if self._active_seeks:
+                song, pos = self._active_seeks[-1]
+                if song is self._player.song:
+                    self._player.emit("seek", song, pos)
+                del self._active_seeks[:]
+        elif message.type == Gst.MessageType.STATE_CHANGED:
+            if message.src is self._playbin.bin:
+                new_state = message.parse_state_changed()[1]
+                if new_state >= Gst.State.PAUSED:
+                    self._refresh_seekable()
+
+                    if self._seek_requests:
+                        song, pos = self._seek_requests[-1]
+                        if song is self._player.song:
+                            self._set_position(song, pos)
+                        del self._seek_requests[:]
+
+    def _set_position(self, song, pos):
+        event = Gst.Event.new_seek(
+            1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
+            Gst.SeekType.SET, pos * Gst.MSECOND, Gst.SeekType.NONE, 0)
+
+        if self._playbin.send_event(event):
+            self._active_seeks.append((song, pos))
+        else:
+            self._seek_requests.append((song, pos))
+
+
 class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
     def PlayerPreferences(self):
@@ -181,15 +327,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         self._volume = 1.0
         self._paused = True
-        self._seekable = False
         self._mute = False
 
         self._in_gapless_transition = False
-        self._active_seeks = []
         self._active_error = False
-        self._last_position = 0
 
         self.bin = None
+        self._seeker = None
         self._int_vol_element = None
         self._ext_vol_element = None
         self._ext_mute_element = None
@@ -247,18 +391,8 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         else:
             print_e("No active pipeline.")
 
-    def _refresh_seekable(self):
-        query = Gst.Query.new_seeking(Gst.Format.TIME)
-        if self.bin and self.bin.query(query):
-            seekable = query.parse_seeking()[1]
-        elif self.song is None:
-            seekable = False
-        else:
-            seekable = True
-
-        if seekable != self._seekable:
-            self._seekable = seekable
-            self.notify("seekable")
+    def _make(self, el, name=None):
+        return Gst.ElementFactory.make(el, name)
 
     def __init_pipeline(self):
         """Creates a gstreamer pipeline. Returns True on success."""
@@ -270,6 +404,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         self.error = False
 
         pipeline = config.get("player", "gst_pipeline")
+        print_d(f"User pipeline (from player.gst_pipeline): {pipeline!r}")
         try:
             pipeline, self._pipeline_desc = GStreamerSink(pipeline)
         except PlayerError as e:
@@ -284,20 +419,21 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             # pipeline supports. As a bonus, this seems to automatically
             # select the highest-precision format supported by the
             # rest of the chain.
-            filt = Gst.ElementFactory.make('capsfilter', None)
+            print_d("Setting up Gstreamer equalizer")
+            filt = self._make('capsfilter', None)
             filt.set_property('caps',
                               Gst.Caps.from_string('audio/x-raw,format=F32LE'))
-            eq = Gst.ElementFactory.make('equalizer-10bands', None)
+            eq = self._make('equalizer-10bands', None)
             self._eq_element = eq
             self.update_eq_values()
-            conv = Gst.ElementFactory.make('audioconvert', None)
-            resample = Gst.ElementFactory.make('audioresample', None)
+            conv = self._make('audioconvert', None)
+            resample = self._make('audioresample', None)
             pipeline = [filt, eq, conv, resample] + pipeline
 
         # playbin2 has started to control the volume through pulseaudio,
         # which means the volume property can change without us noticing.
         # Use our own volume element for now until this works with PA.
-        self._int_vol_element = Gst.ElementFactory.make('volume', None)
+        self._int_vol_element = self._make('volume', None)
         pipeline.insert(0, self._int_vol_element)
 
         # Get all plugin elements and append audio converters.
@@ -306,9 +442,10 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         for plugin in self._get_plugin_elements():
             plugin_pipeline.append(plugin)
             plugin_pipeline.append(
-                Gst.ElementFactory.make('audioconvert', None))
+                self._make('audioconvert', None))
             plugin_pipeline.append(
-                Gst.ElementFactory.make('audioresample', None))
+                self._make('audioresample', None))
+        print_d(f"GStreamer plugin pipeline: {plugin_pipeline}")
         pipeline = plugin_pipeline + pipeline
 
         bufbin = Gst.Bin()
@@ -316,23 +453,15 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             assert element is not None, pipeline
             bufbin.add(element)
 
-        PIPELINE_ERROR = PlayerError(_("Could not create GStreamer pipeline"))
-
         if len(pipeline) > 1:
-            if not link_many(pipeline):
+            try:
+                link_many(pipeline)
+            except OSError as e:
                 print_w("Linking the GStreamer pipeline failed")
-                self._error(PIPELINE_ERROR)
+                self._error(
+                    PlayerError(_("Could not create GStreamer pipeline (%s)" % e)))
                 return False
 
-        # Test to ensure output pipeline can preroll
-        bufbin.set_state(Gst.State.READY)
-        result, state, pending = bufbin.get_state(timeout=STATE_CHANGE_TIMEOUT)
-        if result == Gst.StateChangeReturn.FAILURE:
-            bufbin.set_state(Gst.State.NULL)
-            print_w("Prerolling the GStreamer pipeline failed")
-            self._error(PIPELINE_ERROR)
-            return False
-
         # see if the sink provides a volume property, if yes, use it
         sink_element = pipeline[-1]
         if isinstance(sink_element, Gst.Bin):
@@ -343,7 +472,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self._ext_vol_element = sink_element
 
             # In case we use the sink volume directly we can increase buffering
-            # without affecting the volume change delay too much and safe some
+            # without affecting the volume change delay too much and save some
             # CPU time... (2x default for now).
             if hasattr(sink_element.props, "buffer_time"):
                 sink_element.set_property("buffer-time", 400000)
@@ -371,14 +500,16 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         gpad = Gst.GhostPad.new('sink', pipeline[0].get_static_pad('sink'))
         bufbin.add_pad(gpad)
 
-        self.bin = Gst.ElementFactory.make('playbin', None)
-        assert self.bin
+        bin_ = self._make('playbin', None)
+        assert bin_
 
-        bus = self.bin.get_bus()
+        self.bin = BufferingWrapper(bin_, self)
+        self._seeker = Seeker(self.bin, self)
+
+        bus = bin_.get_bus()
         bus.add_signal_watch()
         self.__bus_id = bus.connect('message', self.__message, self._librarian)
 
-        self.bin = BufferingWrapper(self.bin, self)
         self.__atf_id = self.bin.connect('about-to-finish',
             self.__about_to_finish)
 
@@ -386,12 +517,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         duration = config.getfloat("player", "gst_buffer")
         self._set_buffer_duration(int(duration * 1000))
 
-        # connect playbin to our pluing/volume/eq pipeline
+        # connect playbin to our plugin / volume / EQ pipeline
         self.bin.set_property('audio-sink', bufbin)
 
         # by default playbin will render video -> suppress using fakesink
-        fakesink = Gst.ElementFactory.make('fakesink', None)
-        self.bin.set_property('video-sink', fakesink)
+        vsink = self._make(AudioSinks.FAKE.value, None)
+        self.bin.set_property('video-sink', vsink)
 
         # disable all video/text decoding in playbin
         GST_PLAY_FLAG_VIDEO = 1 << 0
@@ -400,43 +531,24 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         flags &= ~(GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT)
         self.bin.set_property("flags", flags)
 
-        # find the (uri)decodebin after setup and use autoplug-sort
-        # to sort elements like decoders
-        def source_setup(*args):
-            def autoplug_sort(decode, pad, caps, factories):
-                def set_prio(x):
-                    i, f = x
-                    i = {
-                        "mad": -1,
-                        "mpg123audiodec": -2
-                    }.get(f.get_name(), i)
-                    return (i, f)
-                return list(
-                    zip(*sorted(map(set_prio, enumerate(factories)))))[1]
-
-            for e in iter_to_list(self.bin.iterate_recurse):
-                try:
-                    e.connect("autoplug-sort", autoplug_sort)
-                except TypeError:
-                    pass
-                else:
-                    break
-        self.bin.connect("source-setup", source_setup)
-
-        if self.has_external_volume:
-            # ReplayGain information gets lost when destroying
-            self._reset_replaygain()
-        else:
+        if not self.has_external_volume:
             # Restore volume/ReplayGain and mute state
-            self.volume = self._volume
+            self.props.volume = self._volume
             self.mute = self._mute
 
+        # ReplayGain information gets lost when destroying
+        self._reset_replaygain()
+
         if self.song:
-            self.bin.set_property('uri', self.song("~uri"))
+            self._set_uri(self.song("~uri"))
 
         return True
 
+    def _set_uri(self, uri: str) -> None:
+        self.bin.set_property("uri", uri2gsturi(uri))
+
     def __destroy_pipeline(self):
+        print_d("Destroying Gstreamer pipeline")
         self._remove_plugin_elements()
 
         if self.__bus_id:
@@ -449,6 +561,11 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self.bin.disconnect(self.__atf_id)
             self.__atf_id = None
 
+        if self._seeker is not None:
+            self._seeker.destroy()
+            self._seeker = None
+            self.notify("seekable")
+
         if self.bin:
             self.bin.set_state(Gst.State.NULL)
             self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
@@ -457,8 +574,6 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             self.bin = None
 
         self._in_gapless_transition = False
-        self._last_position = 0
-        self._active_seeks = []
 
         self._ext_vol_element = None
         self._int_vol_element = None
@@ -492,13 +607,13 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             gerror, debug_info = message.parse_error()
             message = u""
             if gerror:
-                message = util.gdecode(gerror.message).rstrip(".")
+                message = gerror.message.rstrip(".")
             details = None
             if debug_info:
                 # strip the first line, not user friendly
                 debug_info = "\n".join(debug_info.splitlines()[1:])
                 # can contain paths, so not sure if utf-8 in all cases
-                details = util.gdecode(debug_info)
+                details = debug_info
             self._error(PlayerError(message, details))
         elif message.type == Gst.MessageType.STATE_CHANGED:
             # pulsesink doesn't notify a volume change on startup
@@ -507,20 +622,10 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 self.notify("volume")
             if message.src is self._ext_mute_element:
                 self.notify("mute")
-
-            if message.src is self.bin.bin:
-                new_state = message.parse_state_changed()[1]
-                if new_state >= Gst.State.PAUSED:
-                    self._refresh_seekable()
         elif message.type == Gst.MessageType.STREAM_START:
             if self._in_gapless_transition:
                 print_d("Stream changed")
                 self._end(False)
-        elif message.type == Gst.MessageType.ASYNC_DONE:
-            if self._active_seeks:
-                song, pos = self._active_seeks.pop(0)
-                if song is self.song:
-                    self.emit("seek", song, pos)
         elif message.type == Gst.MessageType.ELEMENT:
             message_name = message.get_structure().get_name()
 
@@ -559,7 +664,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         format_desc = get_description(message)
         title = _(u"No GStreamer element found to handle media format")
         error_details = _(u"Media format: %(format-description)s") % {
-            "format-description": util.gdecode(format_desc)}
+            "format-description": format_desc}
 
         def install_done_cb(plugins_return, *args):
             print_d("Gstreamer plugin install return: %r" % plugins_return)
@@ -608,6 +713,11 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             print_d("multisong: ignore about to finish")
             return
 
+        # mod + gapless deadlocks
+        # https://github.com/quodlibet/quodlibet/issues/2780
+        if isinstance(self.song, ModFile):
+            return
+
         if config.getboolean("player", "gst_disable_gapless"):
             print_d("Gapless disabled")
             return
@@ -638,10 +748,10 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
             # in the mainloop before our function gets scheduled.
             # In this case abort and do nothing, which results
             # in a non-gapless transition.
-            print_d("About to finish (async): %s" % e)
+            print_e("About to finish (async): %s" % e)
             return
         except MainRunnerAbortedError as e:
-            print_d("About to finish (async): %s" % e)
+            print_e("About to finish (async): %s" % e)
             return
         except MainRunnerError:
             util.print_exc()
@@ -649,11 +759,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         if uri is not None:
             print_d("About to finish (async): setting uri")
-            playbin.set_property('uri', uri)
+            self._set_uri(uri)
         print_d("About to finish (async): done")
 
     def stop(self):
-        super(GStreamerPlayer, self).stop()
+        super().stop()
+        print_d("Stop playing")
         self.__destroy_pipeline()
 
     def do_get_property(self, property):
@@ -673,7 +784,9 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                 self._mute = self._ext_mute_element.get_property("mute")
             return self._mute
         elif property.name == "seekable":
-            return self._seekable
+            if self._seeker is not None:
+                return self._seeker.seekable
+            return False
         else:
             raise AttributeError
 
@@ -711,20 +824,9 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         """Return the current playback position in milliseconds,
         or 0 if no song is playing."""
 
-        p = self._last_position
-        if self.song and self.bin:
-            # While we are actively seeking return the last wanted position.
-            # query_position() returns 0 while in this state
-            if self._active_seeks:
-                return self._active_seeks[-1][1]
-
-            ok, p = self.bin.query_position(Gst.Format.TIME)
-            if ok:
-                p //= Gst.MSECOND
-                # During stream seeking querying the position fails.
-                # Better return the last valid one instead of 0.
-                self._last_position = p
-        return p
+        if self._seeker:
+            return self._seeker.get_position()
+        return 0
 
     @property
     def paused(self):
@@ -783,33 +885,22 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
     def seek(self, pos):
         """Seek to a position in the song, in milliseconds."""
+
         # Don't allow seeking during gapless. We can't go back to the old song.
         if not self.song or self._in_gapless_transition:
             return
 
         if self.__init_pipeline():
-            # ensure any pending state changes have completed and we have
-            # at least paused state, so we can seek
-            state = self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)[1]
-            if state < Gst.State.PAUSED:
-                self.bin.set_state(Gst.State.PAUSED)
-                self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
+            self._seeker.set_position(pos)
 
-            pos = max(0, int(pos))
-            gst_time = pos * Gst.MSECOND
-            event = Gst.Event.new_seek(
-                1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
-                Gst.SeekType.SET, gst_time, Gst.SeekType.NONE, 0)
-            if self.bin.send_event(event):
-                # to get a good estimate for when get_position fails
-                self._last_position = pos
-                # For cases where we get the position directly after
-                # a seek and the seek is not done, GStreamer returns
-                # a valid 0 position. To prevent this we try to emit seek only
-                # after it is done. Every flushing seek will trigger
-                # an async_done message on the bus, so we queue the seek
-                # event here and emit in the bus message callback.
-                self._active_seeks.append((self.song, pos))
+    def sync(self, timeout):
+        if self.bin is not None:
+            # XXX: This is flaky, try multiple times
+            for i in range(5):
+                self.bin.get_state(Gst.SECOND * timeout)
+                # we have some logic in the main loop, so iterate there
+                while GLib.MainContext.default().iteration(False):
+                    pass
 
     def _end(self, stopped, next_song=None):
         print_d("End song")
@@ -832,9 +923,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
 
         # Then, set up the next song.
         self.song = self.info = current
-        self.emit('song-started', self.song)
 
-        print_d("Next song")
         if self.song is not None:
             if not self._in_gapless_transition:
                 # Due to extensive problems with playbin2, we destroy the
@@ -852,10 +941,17 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                     self.bin.set_state(Gst.State.PLAYING)
         else:
             self.__destroy_pipeline()
-            self.paused = True
 
         self._in_gapless_transition = False
-        self._refresh_seekable()
+
+        if self._seeker is not None:
+            # we could have a gapless transition to a non-seekable -> update
+            self._seeker.reset()
+
+        self.emit('song-started', self.song)
+
+        if self.song is None:
+            self.paused = True
 
     def __tag(self, tags, librarian):
         if self.song and self.song.multisong:
@@ -882,12 +978,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
-        for key, value in iteritems(sanitize_tags(tags, stream=False)):
+        for key, value in sanitize_tags(tags, stream=False).items():
             if self.song.get(key) != value:
                 changed = True
                 self.song[key] = value
 
-        for key, value in iteritems(sanitize_tags(tags, stream=True)):
+        for key, value in sanitize_tags(tags, stream=True).items():
             if new_info.get(key) != value:
                 info_changed = True
                 new_info[key] = value
