@@ -1,19 +1,22 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 
 from gi.repository import Gtk, GObject, Pango
+from senf import fsnative
 
+from quodlibet import ngettext, _
 from quodlibet import config
 from quodlibet import formats
 from quodlibet import qltk
 from quodlibet import app
 
+from quodlibet.qltk.appwindow import AppWindow
 from quodlibet.formats import AudioFileError
 from quodlibet.plugins import PluginManager
 from quodlibet.qltk.delete import trash_files, TrashMenuItem
@@ -24,19 +27,21 @@ from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
 from quodlibet.qltk.menubutton import MenuButton
-from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.about import AboutDialog
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
 from quodlibet.qltk.x import Align, SeparatorMenuItem, ConfigRHPaned, \
     Button, SymbolicIconImage, MenuItem
-from quodlibet.qltk.window import PersistentWindowMixin, Window, UniqueWindow
+from quodlibet.qltk.window import PersistentWindowMixin, Window
+from quodlibet.qltk.msg import CancelRevertSave
+from quodlibet.qltk.prefs import PreferencesWindow as QLPreferencesWindow
 from quodlibet.qltk import Icons
+from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.path import mtime, normalize_path
-from quodlibet.util import connect_obj, connect_destroy
+from quodlibet.util import connect_obj, connect_destroy, format_int_locale
 from quodlibet.update import UpdateDialog
 
 
-class ExFalsoWindow(Window, PersistentWindowMixin):
+class ExFalsoWindow(Window, PersistentWindowMixin, AppWindow):
 
     __gsignals__ = {
         'changed': (GObject.SignalFlags.RUN_LAST, None, (object,)),
@@ -49,7 +54,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         PluginManager.instance.register_handler(cls.pm)
 
     def __init__(self, library, dir=None):
-        super(ExFalsoWindow, self).__init__(dialog=False)
+        super().__init__(dialog=False)
         self.set_title("Ex Falso")
         self.set_default_size(750, 475)
         self.enable_window_tracking("exfalso")
@@ -117,7 +122,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
         l.set_ellipsize(Pango.EllipsizeMode.END)
         bbox.pack_start(l, True, True, 0)
 
-        fs = MainFileSelector()
+        self._fs = fs = MainFileSelector()
 
         vb.pack_start(fs, True, True, 0)
         vb.pack_start(Align(bbox, border=6), False, True, 0)
@@ -168,15 +173,23 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
     def set_as_osx_window(self, osx_app):
         osx_app.set_menu_bar(self._dummy_osx_menu_bar)
 
-    def get_osx_is_persistent(self):
+    def get_is_persistent(self):
         return False
 
+    def open_file(self, filename):
+        assert isinstance(filename, fsnative)
+
+        if not os.path.isdir(filename):
+            return False
+
+        self._fs.go_to(filename)
+
     def set_pending(self, button, *excess):
         self.__save = button
 
     def __pre_selection_changed(self, view, event, fs, nb):
         if self.__save:
-            resp = qltk.CancelRevertSave(self).run()
+            resp = CancelRevertSave(self).run()
             if resp == Gtk.ResponseType.YES:
                 self.__save.clicked()
             elif resp == Gtk.ResponseType.NO:
@@ -224,7 +237,7 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
             count = len(model or [])
         else:
             count = len(rows)
-        label.set_text(ngettext("%d song", "%d songs", count) % count)
+        label.set_text(numeric_phrase("%d song", "%d songs", count))
 
         for row in rows:
             filename = model[row][0]
@@ -240,44 +253,35 @@ class ExFalsoWindow(Window, PersistentWindowMixin):
                 files.append(song)
             else:
                 files.append(formats.MusicFile(filename))
-        files = filter(None, files)
+        files = list(filter(None, files))
         if len(files) == 0:
             self.set_title("Ex Falso")
         elif len(files) == 1:
             self.set_title("%s - Ex Falso" % files[0].comma("title"))
         else:
+            params = ({'title': files[0].comma("title"),
+                       'count': format_int_locale(len(files) - 1)})
             self.set_title(
                 "%s - Ex Falso" %
-                (ngettext("%(title)s and %(count)d more",
-                          "%(title)s and %(count)d more",
-                          len(files) - 1) % (
-                {'title': files[0].comma("title"), 'count': len(files) - 1})))
+                (ngettext("%(title)s and %(count)s more",
+                          "%(title)s and %(count)s more", len(files) - 1)
+                 % params))
         self.__library.add(files)
         self.emit('changed', files)
 
 
-class PreferencesWindow(UniqueWindow):
+class PreferencesWindow(QLPreferencesWindow):
     def __init__(self, parent):
         if self.is_not_unique():
             return
-        super(PreferencesWindow, self).__init__()
+        super().__init__()
         self.set_title(_("Ex Falso Preferences"))
         self.set_border_width(12)
         self.set_resizable(False)
-        self.set_transient_for(parent)
+        self.set_transient_for(qltk.get_top_parent(parent))
 
-        vbox = Gtk.VBox(spacing=6)
-        hb = Gtk.HBox(spacing=6)
-        e = UndoEntry()
-        e.set_text(config.get("editing", "split_on"))
-        e.connect('changed', self.__changed, 'editing', 'split_on')
-        l = Gtk.Label(label=_("Split _on:"))
-        l.set_use_underline(True)
-        l.set_mnemonic_widget(e)
-        hb.pack_start(l, False, True, 0)
-        hb.pack_start(e, True, True, 0)
-        vbox.pack_start(hb, False, True, 0)
-        f = qltk.Frame(_("Tag Editing"), child=vbox)
+        tagging = self.Tagging()
+        f = qltk.Frame(_("Tag Editing"), child=(tagging.tag_editing_vbox()))
 
         close = Button(_("_Close"), Icons.WINDOW_CLOSE)
         connect_obj(close, 'clicked', lambda x: x.destroy(), self)
@@ -295,8 +299,5 @@ class PreferencesWindow(UniqueWindow):
         connect_obj(self, 'destroy', PreferencesWindow.__destroy, self)
         self.get_child().show_all()
 
-    def __changed(self, entry, section, name):
-        config.set(section, name, entry.get_text())
-
     def __destroy(self):
         config.save()
