@@ -1,12 +1,15 @@
-# -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
 #           2012 Christoph Reiter
 #           2014 Jan Path
-#      2011-2016 Nick Boultbee
+#      2011-2020 Nick Boultbee
+#           2018 David Morris
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+from typing import List, Tuple
 
 from gi.repository import Gtk, GLib, Gdk, GObject
 from senf import uri2fsn
@@ -26,12 +29,12 @@ from quodlibet.qltk.views import AllTreeView, DragScroll
 from quodlibet.qltk.ratingsmenu import ConfirmRateMultipleDialog
 from quodlibet.qltk.songmodel import PlaylistModel
 from quodlibet.qltk import Icons
+from quodlibet.qltk.util import GSignals
 from quodlibet.qltk.delete import trash_songs
 from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
 from quodlibet.qltk.x import SeparatorMenuItem
-from quodlibet.qltk.songlistcolumns import create_songlist_column
+from quodlibet.qltk.songlistcolumns import create_songlist_column, SongListColumn
 from quodlibet.util import connect_destroy
-from quodlibet.compat import xrange, string_types, iteritems, listfilter
 
 
 DND_QL, DND_URI_LIST = range(2)
@@ -42,7 +45,7 @@ class SongSelectionInfo(GObject.Object):
     Songs which get included in the status bar summary.
 
     The `changed` signal gets fired after any of the songs in the
-    selection or the selection it self has changed.
+    selection or the selection itself has changed.
     The signal is async.
 
     Two selection states:
@@ -60,13 +63,13 @@ class SongSelectionInfo(GObject.Object):
             ignore it for now.
     """
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         # changed(songs:list)
         'changed': (GObject.SignalFlags.RUN_LAST, None, (object,))
     }
 
     def __init__(self, songlist):
-        super(SongSelectionInfo, self).__init__()
+        super().__init__()
 
         self.__idle = None
         self.__songlist = songlist
@@ -121,20 +124,8 @@ class SongSelectionInfo(GObject.Object):
 def get_columns():
     """Gets the list of songlist column headings"""
 
-    if config.has_option("settings", "columns"):
-        columns = config.getstringlist(
-            "settings", "columns", const.DEFAULT_COLUMNS)
-    else:
-        # migrate old settings
-        try:
-            columns = config.get("settings", "headers").split()
-        except config.Error:
-            columns = const.DEFAULT_COLUMNS
-        else:
-            config.remove_option("settings", "headers")
-            set_columns(columns)
-            config.setstringlist("settings", "columns", columns)
-
+    columns = config.getstringlist("settings", "columns",
+                                   const.DEFAULT_COLUMNS)
     if "~current" in columns:
         columns.remove("~current")
     return columns
@@ -164,9 +155,9 @@ def get_sort_tag(tag):
         tag = "album"
 
     if "<" in tag:
-        for key, value in iteritems(replace_order):
+        for key, value in replace_order.items():
             tag = tag.replace("<%s>" % key, "<%s>" % value)
-        for key, value in iteritems(TAG_TO_SORT):
+        for key, value in TAG_TO_SORT.items():
             tag = tag.replace("<%s>" % key,
                                "<{1}|<{1}>|<{0}>>".format(key, value))
         tag = Pattern(tag).format
@@ -196,7 +187,7 @@ def header_tag_split(header):
         return util.tagsplit(header)
 
 
-class SongListDnDMixin(object):
+class SongListDnDMixin:
     """DnD support for the SongList class"""
 
     def setup_drop(self, library):
@@ -271,7 +262,17 @@ class SongListDnDMixin(object):
                 return
 
             qltk.selection_set_songs(sel, songs)
-            if ctx.get_actions() & Gdk.DragAction.MOVE:
+
+            # DEM 2018/05/25: The below check is a deliberate repetition of
+            # code in the drag-motion signal handler.  In MacOS/Quartz, the
+            # context action is not propogated between event handlers for
+            # drag-motion and drag-data-get using "ctx.get_actions()".  It is
+            # unclear if this is a bug or expected behavior.  Regardless, the
+            # context widget information is the same so identical behavior can
+            # be achieved by simply using the same widget check as in the move
+            # action.
+            if Gtk.drag_get_source_widget(ctx) == self and \
+                    not self.__force_copy:
                 self.__drag_iters = list(map(model.get_iter, paths))
             else:
                 self.__drag_iters = []
@@ -292,7 +293,8 @@ class SongListDnDMixin(object):
                 except ValueError:
                     return None
 
-            filenames = listfilter(None, map(to_filename, sel.get_uris()))
+            filenames = list(filter(None, map(
+                to_filename, sel.get_uris())))
             move = False
         else:
             Gtk.drag_finish(ctx, False, False, etime)
@@ -305,7 +307,7 @@ class SongListDnDMixin(object):
             elif filename not in library:
                 to_add.append(library.librarian[filename])
         library.add(to_add)
-        songs = listfilter(None, map(library.get, filenames))
+        songs = list(filter(None, map(library.get, filenames)))
         if not songs:
             Gtk.drag_finish(ctx, bool(not filenames), False, etime)
             return
@@ -356,12 +358,12 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                util.InstanceTracker):
     # A TreeView containing a list of songs.
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         # changed(songs:list)
         'orders-changed': (GObject.SignalFlags.RUN_LAST, None, [])
     }
 
-    headers = [] # The list of current headers.
+    headers: List[str] = [] # The list of current headers.
     star = list(Query.STAR)
     sortable = True
 
@@ -394,7 +396,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
     def __init__(self, library, player=None, update=False,
                  model_cls=PlaylistModel):
-        super(SongList, self).__init__()
+        super().__init__()
         self._register_instance(SongList)
         self.set_model(model_cls())
         self.info = SongSelectionInfo(self)
@@ -477,7 +479,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
                     dont_reverse = True
                     break
 
-        # set the inidicators
+        # set the indicators
         default_order = Gtk.SortType.ASCENDING
         reversed_ = False
         for c in self.get_columns():
@@ -579,7 +581,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
             value = model.get_value(iter)(column.header_name)
-            if not isinstance(value, string_types):
+            if not isinstance(value, str):
                 continue
             elif key in value.lower() or key in value:
                 return False
@@ -624,8 +626,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             precision = config.RATINGS.precision
             count = int(float(cellx - 5) / width) + 1
             rating = max(0.0, min(1.0, count * precision))
-            if (rating <= precision and
-                    song("~#rating") == precision):
+            if rating <= precision and song("~#rating") == precision:
                 rating = 0.0
             self.__set_rating(rating, [song], librarian)
 
@@ -683,7 +684,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             model.row_changed(path, iter_)
 
     def __columns_changed(self, *args):
-        headers = map(lambda h: h.header_name, self.get_columns())
+        headers = [h.header_name for h in self.get_columns()]
         SongList.set_all_column_headers(headers)
         SongList.headers = headers
 
@@ -728,7 +729,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         SongList.star = star
 
     def set_model(self, model):
-        super(SongList, self).set_model(model)
+        super().set_model(model)
         self.set_search_column(0)
 
     def clear(self):
@@ -749,16 +750,24 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
     def _sort_songs(self, songs):
         """Sort passed songs in place based on the column sort orders"""
 
+        order = self.get_sort_orders()
+        if not order:
+            return
+        for key, reverse in self.__get_song_sort_key_func(order):
+            songs.sort(key=key, reverse=reverse)
+
+    def __get_song_sort_key_func(self, order):
         last_tag = None
         last_order = None
         first = True
-        for tag, reverse in self.get_sort_orders():
+        key_func = []
+        for tag, reverse in order:
             tag = get_sort_tag(tag)
 
             # always sort using the default sort key first
             if first:
                 first = False
-                songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+                key_func.append((lambda s: s.sort_key, reverse))
                 last_order = reverse
                 last_tag = ""
 
@@ -769,10 +778,11 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             last_tag = tag
 
             if tag == "":
-                songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+                key_func.append((lambda s: s.sort_key, reverse))
             else:
                 sort_func = AudioFile.sort_by_func(tag)
-                songs.sort(key=sort_func, reverse=reverse)
+                key_func.append((sort_func, reverse))
+        return key_func
 
     def add_songs(self, songs):
         """Add songs to the list in the right order and position"""
@@ -789,14 +799,9 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             model.append_many(songs)
             return
 
-        # FIXME: Replace with something fast
-        old_songs = self.get_songs()
-        old_songs.extend(songs)
-
-        self._sort_songs(old_songs)
-
-        for index, song in sorted(zip(map(old_songs.index, songs), songs)):
-            model.insert(index, row=[song])
+        for song in songs:
+            insert_iter = self.__find_song_position(song)
+            model.insert_before(insert_iter, row=[song])
 
     def set_songs(self, songs, sorted=False, scroll=True, scroll_select=False):
         """Fill the song list.
@@ -897,17 +902,76 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         selection.selected_foreach(func, None)
         return songs
 
+    def __find_song_position(self, song):
+        """Finds the appropriate position of a song in a sorted song list.
+
+        Returns iter of the song after the given song according to the current
+        sort order.
+
+        Returns None if the correct position is at the end of the song list.
+        """
+
+        model = self.get_model()
+        order = self.get_sort_orders()
+        sort_key_func = list(enumerate(reversed(
+                self.__get_song_sort_key_func(order))))
+        song_sort_keys = [key(song) for i, (key, r) in sort_key_func]
+        i = 0
+        j = len(model)
+        while i < j:
+            mid = (i + j) // 2
+            other_song_iter = model.iter_nth_child(None, mid)
+            other_song = model.get_value(other_song_iter)
+            song_is_lower = False
+            for i, (key, reverse) in sort_key_func:
+                other_key = key(other_song)
+                is_lower = song_sort_keys[i] < other_key
+                is_greater = song_sort_keys[i] > other_key
+                if not reverse and is_lower or reverse and is_greater:
+                    song_is_lower = True
+                    break
+                if not reverse and is_greater or reverse and is_lower:
+                    break
+            if song_is_lower:
+                j = mid
+            else:
+                i = mid + 1
+        if i < len(model):
+            return model.iter_nth_child(None, i)
+        return None
+
+    def __find_iters_in_selection(self, songs):
+        model, rows = self.get_selection().get_selected_rows()
+        rows = rows or []
+        iters = [model[r].iter for r in rows if model[r][0] in songs]
+        complete = len(iters) == len(songs)
+        return (iters, complete)
+
     def __song_updated(self, librarian, songs):
         """Only update rows that are currently displayed.
         Warning: This makes the row-changed signal useless.
         """
 
+        model = self.get_model()
+        if config.getboolean("song_list", "auto_sort") and self.is_sorted():
+            iters, complete = self.__find_iters_in_selection(songs)
+            if not complete:
+                iters = model.find_all(songs)
+
+            rows = [Gtk.TreeRowReference.new(model, model.get_path(i))
+                    for i in iters]
+
+            for row in rows:
+                iter = model.get_iter(row.get_path())
+                song = model.get_value(iter)
+                insert_iter = self.__find_song_position(song)
+                model.move_before(iter, insert_iter)
+
         vrange = self.get_visible_range()
         if vrange is None:
             return
         (start,), (end,) = vrange
-        model = self.get_model()
-        for path in xrange(start, end + 1):
+        for path in range(start, end + 1):
             row = model[path]
             if row[0] in songs:
                 model.row_changed(row.path, row.iter)
@@ -916,7 +980,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         window = qltk.get_top_parent(self)
         filter_ = window.browser.active_filter
         if callable(filter_):
-            self.add_songs(listfilter(filter_, songs))
+            self.add_songs(list(filter(filter_, songs)))
 
     def __song_removed(self, librarian, songs, player):
         # The player needs to be called first so it can ge the next song
@@ -926,22 +990,22 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             for song in songs:
                 player.remove(song)
 
+        model = self.get_model()
+
         # The selected songs are removed from the library and should
         # be removed from the view.
 
-        if not len(self.get_model()):
+        if not len(model):
             return
 
         songs = set(songs)
 
         # search in the selection first
         # speeds up common case: select songs and remove them
-        model, rows = self.get_selection().get_selected_rows()
-        rows = rows or []
-        iters = [model[r].iter for r in rows if model[r][0] in songs]
+        iters, complete = self.__find_iters_in_selection(songs)
 
         # if not all songs were in the selection, search the whole view
-        if len(iters) != len(songs):
+        if not complete:
             iters = model.find_all(songs)
 
         self.remove_iters(iters)
@@ -1040,7 +1104,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         self.handler_unblock(self.__csig)
 
-    def __getmenu(self, column):
+    def _menu(self, column: SongListColumn) -> Gtk.Menu:
         menu = Gtk.Menu()
 
         def selection_done_cb(menu):
@@ -1048,16 +1112,16 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
 
         menu.connect('selection-done', selection_done_cb)
 
-        current = SongList.headers[:]
-        current_set = set(current)
+        current_set = set(SongList.headers)
 
-        def tag_title(tag):
+        def tag_title(tag: str):
             if "<" in tag:
                 return util.pattern(tag)
             return util.tag(tag)
-        current = zip(map(tag_title, current), current)
+        current = [(tag_title(c), c) for c in SongList.headers]
 
-        def add_header_toggle(menu, pair, active, column=column):
+        def add_header_toggle(menu: Gtk.Menu, pair: Tuple[str, str], active: bool,
+                              column: SongListColumn = column):
             header, tag = pair
             item = Gtk.CheckMenuItem(label=header)
             item.tag = tag
@@ -1074,19 +1138,21 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         sep.show()
         menu.append(sep)
 
-        trackinfo = """title genre ~title~version ~#track
-            ~#playcount ~#skipcount ~rating ~#length ~playlists""".split()
+        trackinfo = """title genre comment ~title~version ~#track
+            ~#playcount ~#skipcount ~rating ~#length ~playlists
+            bpm initialkey""".split()
         peopleinfo = """artist ~people performer arranger author composer
             conductor lyricist originalartist""".split()
         albuminfo = """album ~album~discsubtitle labelid ~#disc ~#discs
             ~#tracks albumartist""".split()
         dateinfo = """date originaldate recordingdate ~year ~originalyear
             ~#laststarted ~#lastplayed ~#added ~#mtime""".split()
-        fileinfo = """~format ~#bitrate ~#filesize ~filename ~basename ~dirname
-            ~uri ~codec ~encoding ~#channels""".split()
+        fileinfo = """~format ~#bitdepth ~#bitrate ~#filesize
+            ~filename ~basename ~dirname ~uri ~codec ~encoding ~#channels
+            ~#samplerate""".split()
         copyinfo = """copyright organization location isrc
             contact website""".split()
-        all_headers = sum(
+        all_headers: List[str] = sum(
             [trackinfo, peopleinfo, albuminfo, dateinfo, fileinfo, copyinfo],
             [])
 
@@ -1175,7 +1241,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         if event is not None and event.button != Gdk.BUTTON_SECONDARY:
             return False
 
-        menu = self.__getmenu(column)
+        menu = self._menu(column)
         menu.attach_to_widget(self, None)
 
         if event:
