@@ -1,21 +1,21 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
 #           2013 Christoph Reiter
+#           2020 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from __future__ import absolute_import
 
 from collections import MutableSequence, defaultdict
 
-from quodlibet.compat import listkeys
 from .misc import total_ordering
 
 
 @total_ordering
-class DictMixin(object):
+class DictMixin:
     """Implement the dict API using keys() and __*item__ methods.
 
     Similar to UserDict.DictMixin, this takes a class that defines
@@ -58,7 +58,7 @@ class DictMixin(object):
         return iter(self.items())
 
     def clear(self):
-        for key in listkeys(self):
+        for key in list(self.keys()):
             del self[key]
 
     def pop(self, key, *args):
@@ -76,7 +76,7 @@ class DictMixin(object):
 
     def popitem(self):
         try:
-            key = listkeys(self)[0]
+            key = list(self.keys())[0]
             return key, self.pop(key)
         except IndexError:
             raise KeyError("dictionary is empty")
@@ -124,7 +124,7 @@ class DictMixin(object):
 class DictProxy(DictMixin):
     def __init__(self, *args, **kwargs):
         self.__dict = {}
-        super(DictProxy, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def __getitem__(self, key):
         return self.__dict[key]
