@@ -31,7 +31,7 @@ from quodlibet.library.libraries import FileLibrary
 from tests.test_browsers_search import SONGS
 
 
-class ConfigSetupMixin(object):
+class ConfigSetupMixin:
     def setUp(self):
         quodlibet.config.init()
 
@@ -39,7 +39,7 @@ class ConfigSetupMixin(object):
         quodlibet.config.quit()
 
 
-class TParsePlaylistMixin(object):
+class TParsePlaylistMixin:
 
     def test_parse_empty(self):
         with temp_filename() as name:
@@ -111,8 +111,8 @@ class TPlaylistIntegration(TestCase):
 
     def setUp(self):
         quodlibet.config.init()
-        self.lib = quodlibet.browsers.search.library = FileLibrary()
-        quodlibet.browsers.search.library.librarian = SongLibrarian()
+        self.lib = quodlibet.browsers.tracks.library = FileLibrary()
+        quodlibet.browsers.tracks.library.librarian = SongLibrarian()
         for af in self.SONGS:
             af.sanitize()
         self.lib.add(self.SONGS)
@@ -199,11 +199,19 @@ class TPlaylistsBrowser(TestCase):
 
         PlaylistsBrowser.init(self.lib)
 
-        self.bar = PlaylistsBrowser(self.lib)
+        self.bar = PlaylistsBrowser(self.lib, self.MockConfirmerAccepting)
         self.bar.connect('songs-selected', self._expected)
         self.bar._select_playlist(self.bar.playlists()[0])
         self.expected = None
 
+        # Uses the declining confirmer.
+        self.bar_decline = PlaylistsBrowser(self.lib, self.MockConfirmerDeclining)
+        self.bar_decline.connect('songs-selected', self._expected_decline)
+        self.bar_decline._select_playlist(self.bar_decline.playlists()[0])
+        # Note that _do() uses self.expected, but _do() is not called by the
+        # testcase for declining the prompt. Tests fail with a shared expected.
+        self.expected_decline = None
+
     def tearDown(self):
         self.bar.destroy()
         self.lib.destroy()
@@ -217,6 +225,12 @@ class TPlaylistsBrowser(TestCase):
             self.failUnlessEqual(self.expected, songs)
         self.success = True
 
+    def _expected_decline(self, bar, songs, sort):
+        songs.sort()
+        if self.expected_decline is not None:
+            self.failUnlessEqual(self.expected_decline, songs)
+        self.success = True
+
     def _do(self):
         while Gtk.events_pending():
             Gtk.main_iteration()
@@ -280,17 +294,11 @@ class TPlaylistsBrowser(TestCase):
         qltk.selection_set_songs(sel, [song])
         b._drag_data_get(None, None, sel, DND_QL, None)
 
+    # deletion of playlist tracks
     def test_deletion(self):
-        def a_delete_event():
-            ev = Gdk.Event()
-            ev.type = Gdk.EventType.KEY_PRESS
-            ev.keyval, accel_mod = Gtk.accelerator_parse("Delete")
-            ev.state = Gtk.accelerator_get_default_mod_mask() & accel_mod
-            return ev
-
         b = self.bar
         self._fake_browser_pack(b)
-        event = a_delete_event()
+        event = self.a_delete_event()
         # This is selected in setUp()
         first_pl = b.playlists()[0]
         app.window.songlist.set_songs(first_pl)
@@ -301,6 +309,33 @@ class TPlaylistsBrowser(TestCase):
         self.failUnless(ret, msg="Didn't simulate a delete keypress")
         self.failUnlessEqual(len(first_pl), original_length - 1)
 
+    def test_playlist_deletion_ACCEPT(self):
+        b = self.bar
+        orig_length = len(b.playlists())
+        event = self.a_delete_event()
+        first_pl = b.playlists()[0]
+        second_pl = b.playlists()[1]
+        b._select_playlist(first_pl)
+
+        ret = b._PlaylistsBrowser__key_pressed(b, event)
+        self.failUnless(ret, msg="Didn't simulate a delete keypress")
+        self.failUnlessEqual(len(b.playlists()), orig_length - 1)
+        self.failUnlessEqual(b.playlists()[0], second_pl)
+
+    def test_playlist_deletion_CANCEL(self):
+        b = self.bar_decline
+        orig_length = len(b.playlists())
+        event = self.a_delete_event()
+        first_pl = b.playlists()[0]
+        second_pl = b.playlists()[1]
+        b._select_playlist(first_pl)
+
+        ret = b._PlaylistsBrowser__key_pressed(b, event)
+        self.failUnless(ret, msg="Didn't simulate a delete keypress")
+        self.failUnlessEqual(len(b.playlists()), orig_length)
+        self.failUnlessEqual(b.playlists()[0], first_pl)
+        self.failUnlessEqual(b.playlists()[1], second_pl)
+
     def test_import(self):
         pl_name = u"_€3 œufs à Noël"
         pl = FileBackedPlaylist(_TEMP_DIR, pl_name, None)
@@ -308,7 +343,7 @@ class TPlaylistsBrowser(TestCase):
         pl.write()
         new_fn = os.path.splitext(text2fsn(pl.name))[0] + '.m3u'
         new_path = os.path.join(pl.dir, new_fn)
-        os.rename(pl.filename, new_path)
+        os.rename(pl.path, new_path)
         added = self.bar._import_playlists([new_path], self.lib)
         self.failUnlessEqual(added, 1, msg="Failed to add '%s'" % new_path)
         os.unlink(new_path)
@@ -322,10 +357,38 @@ class TPlaylistsBrowser(TestCase):
             return [song('~filename') for song in songs]
         self.failUnlessEqual(fns(imported.songs), fns(pl.songs))
 
+    @staticmethod
+    def a_delete_event():
+        ev = Gdk.Event()
+        ev.type = Gdk.EventType.KEY_PRESS
+        ev.keyval, accel_mod = Gtk.accelerator_parse("Delete")
+        ev.state = Gtk.accelerator_get_default_mod_mask() & accel_mod
+        return ev
+
     @staticmethod
     def _fake_browser_pack(b):
         app.window.get_child().pack_start(b, True, True, 0)
 
+    class MockConfirmerAccepting:
+
+        RESPONSE_INVOKE = Gtk.ResponseType.YES
+
+        def __init__(self, *args):
+            pass
+
+        def run(self, *args):
+            return self.RESPONSE_INVOKE
+
+    class MockConfirmerDeclining:
+
+        RESPONSE_INVOKE = Gtk.ResponseType.YES
+
+        def __init__(self, *args):
+            pass
+
+        def run(self, *args):
+            return Gtk.ResponseType.CANCEL
+
 
 class TPlaylistUtils(TestCase):
 
