@@ -30,7 +30,7 @@ class TID3ImagesBase(TestCase):
         os.remove(self.filename)
 
 
-class TID3ImagesMixin(object):
+class TID3ImagesMixin:
 
     def test_can_change_images(self):
         self.failUnless(self.KIND(self.filename).can_change_images)
@@ -130,7 +130,7 @@ class TID3FileBase(TestCase):
         os.unlink(self.filename)
 
 
-class TID3FileMixin(object):
+class TID3FileMixin:
 
     def test_optional_POPM_count(self):
         # https://github.com/quodlibet/quodlibet/issues/364
@@ -405,7 +405,7 @@ class TID3FileMixin(object):
 
         f = mutagen.File(self.filename)
         for k in ["track_peak", "track_gain", "album_peak", "album_gain"]:
-            self.failUnless(f["TXXX:replaygain_" + k])
+            self.failUnless(f[f"TXXX:REPLAYGAIN_{k.upper()}"])
 
     def test_foobar2k_rg_caseinsensitive(self):
         f = mutagen.File(self.filename)
@@ -416,13 +416,13 @@ class TID3FileMixin(object):
         self.failUnlessEqual(song("replaygain_track_gain"), "-6 db")
         song.write()
         f = mutagen.File(self.filename)
-        frames = f.tags.getall("TXXX:replaygain_track_gain")
+        frames = f.tags.getall("TXXX:REPLAYGAIN_TRACK_GAIN")
         self.assertTrue(frames)
-        self.assertEqual(frames[0].desc, "replaygain_track_gain")
+        self.assertEqual(frames[0].desc, "REPLAYGAIN_TRACK_GAIN")
         del song["replaygain_track_gain"]
         song.write()
         f = mutagen.File(self.filename)
-        self.assertFalse(f.tags.getall("TXXX:replaygain_track_gain"))
+        self.assertFalse(f.tags.getall("TXXX:REPLAYGAIN_TRACK_GAIN"))
 
     def test_quodlibet_txxx_inval(self):
         # This shouldn't happen in our namespace, but check anyway since
