@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,7 +18,7 @@ class TIconTheme(TestCase):
 
         for i in [
             "io.github.quodlibet.QuodLibet",
-            "exfalso",
+            "io.github.quodlibet.ExFalso",
             "quodlibet-missing-cover"
         ]:
             self.failUnless(theme.has_icon(i))
