@@ -37,12 +37,12 @@ class MissingTranslationsException(Exception):
     def __init__(self, missing):
         msg = ("No reference in POTFILES.in to: " +
                ", ".join(missing))
-        super(MissingTranslationsException, self).__init__(msg)
+        super().__init__(msg)
 
 
 @pytest.mark.skipif(not has_gettext_util(), reason="no gettext")
 def test_potfile_format():
-    with gettextutil.create_pot(PODIR, strict=True) as pot_path:
+    with gettextutil.create_pot(PODIR) as pot_path:
         gettextutil.check_pot(pot_path)
 
 
@@ -69,7 +69,7 @@ class TPot(TestCase):
 
     @classmethod
     def setUpClass(cls):
-        with gettextutil.create_pot(PODIR, strict=True) as pot_path:
+        with gettextutil.create_pot(PODIR) as pot_path:
             cls.pot = polib.pofile(pot_path)
 
     def conclude(self, fails, reason):
@@ -243,7 +243,7 @@ class TPot(TestCase):
         self.conclude(fails, "leading or trailing spaces")
 
 
-class POMixin(object):
+class POMixin:
 
     @pytest.mark.skipif(not has_gettext_util(), reason="no gettext")
     def test_pos(self):
