@@ -120,6 +120,11 @@ class Tunexpand(TestCase):
         path = unexpand(os.path.join(self.d, "la", "la"))
         self.failUnlessEqual(path, os.path.join(self.u, "la", "la"))
 
+    def test_case_insensitive_win(self):
+        if is_win:
+            assert unexpand(self.d.lower()) == "%USERPROFILE%"
+            assert unexpand(self.d.upper()) == "%USERPROFILE%"
+
 
 class Tformat_rating(TestCase):
     def setUp(self):
@@ -730,6 +735,11 @@ class Tescape_filename(TestCase):
         self.assertEqual(result, "abc%C3%A4")
         self.assertTrue(isinstance(result, fsnative))
 
+    def test_safe_chars(self):
+        result = escape_filename('1, 2, and -3', safe=' -')
+        self.assertEqual(result, "1%2C 2%2C and -3")
+        self.assertTrue(isinstance(result, fsnative))
+
 
 @skipIf(is_win, "not on Windows")
 class Tload_library(TestCase):
@@ -987,7 +997,7 @@ class Tcached_property(TestCase):
 
     def test_main(self):
 
-        class A(object):
+        class A:
             @util.cached_property
             def foo(self):
                 return object()
@@ -1002,7 +1012,7 @@ class Tcached_property(TestCase):
 
         def define_class():
 
-            class A(object):
+            class A:
                 @util.cached_property
                 def __foo_(self):
                     return object()
